
/*
 * Ethnocentricity Model
 *
 * "Additive" variant (new REGULAR case), originally coded 3/12/02
 * As per Ethn29 memo, 3/7/02
 *
 * This code version used for paper submitted to SCIENCE, 4/02
 *
 * Additions 5/9/02:
 *	- combat variant and switch
 *  - cost to discrim and switch
 *  - cleaned up code comments for documentation purposes
 *
 */

package edu.brook.ethnoadd;

import java.lang.*;
import java.awt.*;
import java.io.*;

import edu.brook.ascape.model.*;
import edu.brook.ascape.util.*;
import edu.brook.ascape.view.*;

import edu.brook.*;

public class EthnoModel extends ScapeVector {
    //model parameter settings
    protected int latticeWidth = 50; //50 for "regular" case
    protected int latticeHeight = 50; //50 for "regular" case

	protected double mutationRate = 0.005; //.005 "regular" case
    protected double deathRate = 0.10; //0.10 "regular" case
    
    protected int immigrantsPerDay = 1; //1 "regular" case
    
    protected int stopAtIteration = 2000; //2000 "regular" case
    protected boolean restartModel = true; //true

    
    protected int colorSetting = 4; //must be 2,4,6,8 etc.
    
    //The "payoffs"
    protected double baseOTR = 0.12; //0.12 "regular" case
    protected double costOfGiving = .01; //.01 "regular" case
    protected double gainOfReceiving = .03; //.03 "regular" case

	protected boolean combatOn = false; //if on, "D" gives positive payoff to player, negative payoff to partner
									   // (see Eth1001 memo)

	protected boolean discrimCostOn = false;
	protected double discrimCost = 0.03;
	
    //model switches
    protected boolean useMoreColors = true; //true ("reg" case) //2 colors if false, 4 colors if true    
    protected boolean useOneTagOnly = false; //false ("regular" case)
    
    protected boolean halfImmigrant = false; //false ("regular" case)
    
    protected int childNearParentZeroFalse = 2; //2 //anything other than 0 means true
    protected boolean locateChildNearParent = true; //true

    protected boolean noiseOn = false;  //false for "regular" case
    protected double noiseLevel = 0.10;
    
    protected boolean useSpecifiedRandomSeed = false; //false
    protected long specifiedSeed = 1016508794929L; //for "regular" focal case run #1 (EXP104)


    //data output switches
    protected boolean completeDataOutput = false; //iter by iter data for a run
    protected boolean summaryDataOutput = true; //summary data only (cumul., last iter, and last 100 iter avgs)
    protected boolean headerData = true; //header info for run (parameter settings, etc.)
    protected boolean viewsOn = false;
    
    //Data output labels
    protected int experimentNumber = 300; //experiment number for run(s)
    protected String experimentDescr = "Standard case, reruns of 9/02"; //short exp description
    
    protected int runMax = 10; //number of runs for experiment
    
    //constants
    public static final boolean COOPERATE = true;
    public static final boolean DEFECT = false;
    public static final int BLUE = 0;
    public static final int RED = 1;
     
	//internal counters
    protected int[] phenotypeStats = new int[5]; //meet own, meet other, def other, coop own
	protected int[] lasthundredPhenotypeStats = new int[5];
	protected int[] lasthundredGenotypeStats = new int[4];
	protected final int MEETOWN = 0;
	protected final int MEETOTHER = 1;
	protected final int DEFOTHER = 2;
	protected final int COOPOWN = 3;
	protected final int COOPOTHER = 4;
	protected final int CC = 0;
	protected final int CD = 1;
	protected final int DC = 2;
	protected final int DD = 3;

	//test code 5/13/02
	//protected int countBeginning;
	protected int countAfterImm = 0;
	protected int countAfterFission = 0;
	//protected int countAfterDth;
	

    protected int playMode = 0;

	public int runNumber;
	
    ScapeGraph lattice;
    
    ScapeVector agents;
    
    Overhead2DView overheadView;
    
    StatCollectorCond countEthnocentrics;
    StatCollectorCond countTotalDefectors;
    StatCollectorCond countRed;
    StatCollectorCond countBlue;
    StatCollectorCond countGreen;
    StatCollectorCond countYellow;
    StatCollectorCond countCoopBoth;
    StatCollectorCond countCoopOwnDefOther;
    StatCollectorCond countDefOwnCoopOther;
    StatCollectorCond countDefBoth;
    
    //These are the file names and paths for iter-by-iter data output
    File outputfile = new File("c:/Docs/axelrod/Ethnocentrism Model/data and analysis/experiment "+experimentNumber+" batchData.txt");
	File outputfile2 = new File("c:/Docs/axelrod/Ethnocentrism Model/data and analysis/experiment "+experimentNumber+" batchRunFile.txt");
	
	//This is file name and path for Summary data output
	//A header file named "about.txt" will also be created in this directory
	//containing the parameters used for all the experiment runs
	String summaryDataFile = ("c:/Docs/axelrod/Ethnocentrism Model/data and analysis/experiment "+experimentNumber+".txt");
	
    ChartView chartView;
    ChartView behaviorChart;
    


    public void createScape() { //this initializes memory for scape vectors, etc.
        super.createScape();
        
        //This sets up a lattice (using Von N geometry) of size as specified
        //above, and fills it with cells of type "EthnoCell" to host players 
        lattice = new ScapeArray2DVonNeumann(); //VonNeumann()
        lattice.setPrototypeAgent(new EthnoCell());
        lattice.setExtent(new Coordinate2DDiscrete(latticeWidth, latticeHeight));
        
        //This sets up a population of players, who live on the lattice
        EthnoPlayer player = new edu.brook.ethnoadd.EthnoPlayer();
        player.setHostScape(lattice);
		agents = new ScapeVector();
		agents.setName("Players");
		agents.setPrototypeAgent(player);
        agents.setExecutionOrder(Scape.RULE_ORDER);
				
		addAgent(lattice);
		addAgent(agents);
		
		//This code instructs the model to stop automatically at the specified iteration
		try {
    		setStopPeriod(stopAtIteration);
    	}
    	catch (SpatialTemporalException e) {
    	    System.out.println("Bad start/stop periods: " + e);
    	}
	
	    //This code instructs the model to restart automatically if the number of
	    //runs completed so far is less than the desired total
	    setAutoRestart(restartModel);
	    
	    //For specifying a random seed
	    if (useSpecifiedRandomSeed) {
	        setRandomSeed(specifiedSeed);
	    }
	    
	    //This code initializes basic statistics counters
	    for (int i=0;i<phenotypeStats.length;i++) {
	        phenotypeStats[i] = 0;
	        lasthundredPhenotypeStats[i] = 0;
	    }
	    
	    for (int i=0;i<4;i++) {
	        lasthundredGenotypeStats[i] = 0;
	    }
	        
	    runNumber = 0;
	    
	    //This code specifies the output syntax for the header file
	    if (headerData) {
	     try {
      	        FileOutputStream fos = new FileOutputStream("c:/Docs/axelrod/Ethnocentrism Model/data and analysis/experiment "+experimentNumber+" about.txt", false);		  
                PrintWriter out = new PrintWriter(fos);
                out.print("EXP");
                out.println(experimentNumber);
	            out.println("------------------------------");                
                out.println(experimentDescr);
                out.println("------------------------------");
                
                out.print("latticeWidth ");
                out.println(latticeWidth);
                out.print("latticeHeight ");
                out.println(latticeHeight);
                out.print("mutationRate ");
                out.println(mutationRate);
                out.print("deathRate ");
                out.println(deathRate);
                out.print("immigrantsPerDay ");
                out.println(immigrantsPerDay);
                out.print("baseOTR ");
                out.println(baseOTR);;
                out.print("costOfGiving ");
                out.println(costOfGiving);
                out.print("gainOfReceiving ");
                out.println(gainOfReceiving);
                out.print("useMoreColors ");
                out.println(useMoreColors);
                out.print("useOneTagOnly ");
                out.println(useOneTagOnly);
                out.print("halfImmigrant ");
                out.println(halfImmigrant);
                out.print("locateChildNearParent ");
                out.println(locateChildNearParent);
                
                out.print("noise On");
                out.println(noiseOn);
                
	            out.close();
        	    } catch (FileNotFoundException e) {
		        	System.err.println("FileStreamsTest: "+ e);
	        	} catch (IOException e) {
			        System.err.println("FileStreamsTest: "+ e);
		        }	
		 
	    }
	    
	}
	
	
	
	public void createViews() {
	    super.createViews();
	    
	    
	      /*
            SweepControlView standardExp = new SweepControlView();
            this.addView(standardExp);
            
            SweepLink costOfDonation = new SweepLink();
            costOfDonation.addMember(new SweepDimension(this, "costOfGiving", 0.005, 0.02, 0.005));
        
            SweepLink numberOfColors = new SweepLink();
			numberOfColors.addMember(new SweepDimension(this, "colorSetting", 2, 8, 2));
		
		    SweepLink currentMutationRate = new SweepLink();
            currentMutationRate.addMember(new SweepDimension(this, "mutationRate", 0.0025, 0.01, 0.0025));
            
            standardExp.getSweepGroup().addMember(costOfDonation);
            standardExp.getSweepGroup().addMember(numberOfColors);
            standardExp.getSweepGroup().addMember(currentMutationRate);
            standardExp.getSweepGroup().setRunsPer(10);
            */
	    
	    
	    //STATISTICS COLLECTORS
	    countEthnocentrics = new StatCollectorCond("Ethnocentrics") {
                public boolean meetsCondition(Object object) {
                   return ((((EthnoPlayer) object).strategyToOwn) && (!((EthnoPlayer) object).strategyToOther));              
                }
           };
	    
	    countTotalDefectors = new StatCollectorCond("DefectAgainstAll") {
	            public boolean meetsCondition(Object object) {
	                return ((!((EthnoPlayer) object).strategyToOwn) && (!((EthnoPlayer) object).strategyToOther));
	            }
	            
	    };
	    
	    countRed = new StatCollectorCond("Reds") {
	            public boolean meetsCondition(Object object) {
	                return (((EthnoPlayer) object).tag == 1);
	            }
	    };
	    
	    countBlue = new StatCollectorCond("Blues") {
	            public boolean meetsCondition(Object object) {
	                return (((EthnoPlayer) object).tag == 0);
	            }
	    };
	    
	    countGreen = new StatCollectorCond("Greens") {
	            public boolean meetsCondition(Object object) {
	                return (((EthnoPlayer) object).tag == 2);
	            }
	    };
	    
	    countYellow = new StatCollectorCond("Yellows") {
	            public boolean meetsCondition(Object object) {
	                return (((EthnoPlayer) object).tag == 3);
	            }
	    };
	    
	    
	    //GENOTYPE STATS
	    
  	    countCoopBoth = new StatCollectorCond("CoopBoth") {
	            public boolean meetsCondition(Object object) {
	                return ((((EthnoPlayer) object).strategyToOwn) && ((EthnoPlayer) object).strategyToOther);
	            }
	    };
	    countCoopOwnDefOther = new StatCollectorCond("CoopOwnDefOther") {
	            public boolean meetsCondition(Object object) {
   	                return ((((EthnoPlayer) object).strategyToOwn) && !((EthnoPlayer) object).strategyToOther);
	            }
	    };
	    countDefOwnCoopOther = new StatCollectorCond("DefOwnCoopOther") {
	            public boolean meetsCondition(Object object) {
                    return (!(((EthnoPlayer) object).strategyToOwn) && ((EthnoPlayer) object).strategyToOther);
	            }
	    };
	    countDefBoth = new StatCollectorCond("DefBoth") {
	            public boolean meetsCondition(Object object) {
	                return (!(((EthnoPlayer) object).strategyToOwn) && !((EthnoPlayer) object).strategyToOther);
	            }
	    };

        
	    agents.addStatCollector(countEthnocentrics);	
	    agents.addStatCollector(countTotalDefectors);
	    agents.addStatCollector(countRed);
	    agents.addStatCollector(countBlue);
	    agents.addStatCollector(countGreen);
	    agents.addStatCollector(countYellow);
	    agents.addStatCollector(countCoopBoth);
	    agents.addStatCollector(countCoopOwnDefOther);
	    agents.addStatCollector(countDefOwnCoopOther);
	    agents.addStatCollector(countDefBoth);
	    
	    
	    //SET UP BASIC ITERATION BY ITERATION DATA OUTPUT
	    if (completeDataOutput) {
	        DataOutputView dataRecord = new DataOutputView();
            addView(dataRecord);        
        
            try{
                dataRecord.setPeriodFile(outputfile);
                dataRecord.setRunFile(outputfile2);
            }
            catch (IOException e) { System.out.println(e); }
       
	    }
	    
	    
	    //SET UP VIEWS
	    if (viewsOn) {
	        chartView = new ChartView();
	        agents.addView(chartView);
	        chartView.setName("Types of Agent");
	    
	        chartView.addSeries("Count Ethnocentrics", Color.green);
	        chartView.addSeries("Count DefectAgainstAll", Color.magenta);
	    
	        overheadView = new Overhead2DView();
            overheadView.setCellSize(10);
            overheadView.setName("Agent Lattice View");
            lattice.addView(overheadView);	
        
        
            //This code sets up the coloring of the agents, allowing for a
            //"light" and "dark" variant of each color
            final ColorFeatureGradiated playerColor = new ColorFeatureGradiated("PlayerWealth");
            playerColor.setDataPoint(new UnitIntervalDataPoint() {
                public double getValue(Object object) {
                    
                    double foo = 0.0;
                  
                    //REDS
                    if (((EthnoPlayer) object).tag == 1) {
                            playerColor.setMaximumColor(Color.red);
    	                    playerColor.setMinimumColor(Color.white);
        	                
    	                    if (((EthnoPlayer) object).strategyToOwn) {
        	                    foo = 1.0;
    	                    }
    	                    else {
        	                    foo = 0.5;
    	                    }
        	                
    	            }
    	            
    	            //BLUES
    	            if (((EthnoPlayer) object).tag == 0) {
                            playerColor.setMaximumColor(Color.blue);
    	                    playerColor.setMinimumColor(Color.white);
        	                
    	                    if (((EthnoPlayer) object).strategyToOwn) {
        	                    foo = 1.0;
    	                    }
    	                    else {
        	                    foo = 0.5;
    	                    }
        	                
    	            }
    	            
    	            //TYPE 3 - GREENS or MAGENTAS
    	            if (((EthnoPlayer) object).tag == 2) {
                            /*
                            playerColor.setMaximumColor(Color.green);
    	                    playerColor.setMinimumColor(Color.white);
    	                    */
        	                
    	                    playerColor.setMaximumColor(Color.magenta);
    	                    playerColor.setMinimumColor(Color.white);
        	                
    	                    if (((EthnoPlayer) object).strategyToOwn) {
        	                    foo = 1.0;
    	                    }
    	                    else {
        	                    foo = 0.5;
    	                    }
        	                
    	            }
    	            
    	            //TYPE 4 - YELLOWS or BROWNS
    	            if (((EthnoPlayer) object).tag == 3) {
        	            
        	            
    	                    //orig
    	                    /*
                            playerColor.setMaximumColor(Color.yellow);
    	                    playerColor.setMinimumColor(Color.white);
    	                    */
        	                
    	                    //v3
    	                    /*
    	                    playerColor.setMaximumColor(Color.yellow);
    	                    playerColor.setMinimumColor(Color.white);
    	                    */
        	                
    	                    //v2
        	                
    	                    Color brown = new Color(1, 1, 0);
        	                
    	                    playerColor.setMaximumColor(brown);
    	                    playerColor.setMinimumColor(Color.white);
        	                
        	                
    	                    if (((EthnoPlayer) object).strategyToOwn) {
        	                    foo = 1.0;
    	                    }
    	                    else {
        	                    //v3
    	                        //foo = 0.15;
    	                        //orig
    	                        foo = 0.5;
    	                    }
        	                
    	            }    
                    return (foo);
                }
	        });
    	   
    	   
	    //so, empty cells are light gray
	    //dark dots on dark gray backgrounds are ETHNOCENTRIC
	    //light dots on dark backgrounds are DEFECT WITH ALL
	    //dark dots on light background are UNIVERSAL COOPERATION
	    //light dots on light background are ANTI-ETHNOCENTRIC
    	   
    	   
    	   //This code sets up background cell colors, allowing for
    	   //"light" and "dark" backgrounds
	        final ColorFeatureGradiated cellColor = new ColorFeatureGradiated("PlayerStratVOther");
            cellColor.setDataPoint(new UnitIntervalDataPoint() {
                public double getValue(Object object) {
                    
                    double foo = 0.0;
                    
                    
                    if (((HostCell) object).isAvailable()) {
                        //cellColor.setMaximumColor(Color.black);
                        cellColor.setMaximumColor(Color.lightGray);
                        foo = 1.0; //EMPTY cells are light Gray
                    }
                    else {
                        
                        
                        cellColor.setMaximumColor(Color.lightGray); //dark Gray backgrounds
                        cellColor.setMinimumColor(Color.darkGray); //lightGray backgrounds
                    
                        EthnoPlayer occupant = ((EthnoPlayer) ((HostCell) object).getOccupant());
                                   
                        if (occupant.strategyToOther) {
                            foo = 1.0; //COOP WITH OTHER = light Gray
                        }
                        else {
                            foo = 0.0; //DEF WITH OTHER = dark Gray
                        }                
    	            }
        	        
                    return (foo);
                }
	        });
    	    
            overheadView.setAgentColorFeature(playerColor);
            overheadView.setCellColorFeature(cellColor);
            }//end if viewsOn
            
          
            
          
            
        }
    
        
        
    public void iterateScape() {
        
        
        /*       
        //immigrants arrive
        if (halfImmigrant) {
         
          if (randomIs()) {  
            if (lattice.findRandomUnoccupiedCell() != null) {
                EthnoPlayer immigrant = new edu.brook.ethnoadd.EthnoPlayer();
                agents.addAgent(immigrant);
                immigrant.moveTo((HostCell) lattice.findRandomUnoccupiedCell());
                immigrant.initialize();
        	}
        }
 
        }
        else {
        
        for (int i=0; i<immigrantsPerDay; i++) {
           
            if (lattice.findRandomUnoccupiedCell() != null) {
                EthnoPlayer immigrant = new edu.brook.ethnoadd.EthnoPlayer();
                agents.addAgent(immigrant);
                immigrant.moveTo((HostCell) lattice.findRandomUnoccupiedCell());
                immigrant.initialize();
            }
        }
        
        
        
    }
        
        */
        //all agents iterate
	    super.iterateScape();   
	    
	    //if in last 100 iterations, update appropriate stats
	    if (getIteration() >= (stopAtIteration-100)) {
	        lasthundredGenotypeStats[0] += countCoopBoth.getDataSeries("Count").getValue();
	        lasthundredGenotypeStats[1] += countCoopOwnDefOther.getDataSeries("Count").getValue();
	        lasthundredGenotypeStats[2] += countDefOwnCoopOther.getDataSeries("Count").getValue();
	        lasthundredGenotypeStats[3] += countDefBoth.getDataSeries("Count").getValue();
	    }
	    
	    
	    
	    
	    //System.out.print(getIteration());
		//System.out.print("   ");
		//System.out.print(countAfterImm);
		//System.out.print("   ");
		//System.out.println(countAfterFission);

        
        //countAfterImm = 0;
        //countAfterFission = 0;
        

		
	    
	    //summary data output (last iteration)
	    
	    if (summaryDataOutput) {
	    
	    if (getIteration() == (stopAtIteration)) {
	    
	      //last iteration genotype stats
	      //(not used in current version, but collected in case useful later)
          double dataCoopBoth = countCoopBoth.getDataSeries("Count").getValue();
          double dataCoopOwnDefOther = countCoopOwnDefOther.getDataSeries("Count").getValue();
          double dataDefOwnCoopOther = countDefOwnCoopOther.getDataSeries("Count").getValue();
          double dataDefBoth = countDefBoth.getDataSeries("Count").getValue();

          //last iteration color stats
          //(not used in current version, but collected in case useful later)
          double dataCountRed = countRed.getDataSeries("Count").getValue();
          double dataCountBlue = countBlue.getDataSeries("Count").getValue();
          double dataCountGreen = countGreen.getDataSeries("Count").getValue();
          double dataCountYellow = countYellow.getDataSeries("Count").getValue();
        
          int ALLC = 0;
          int ALLD = 1;
          int ETHNO = 2;
          int ANTIETHNO = 3;
        
          Agent[] theCells = lattice.getAgents();
        
          //avg. Genotypes over last 100 iterations
          for (int i=0;i<4;i++) {
            lasthundredGenotypeStats[i] = lasthundredGenotypeStats[i]/100;
          }
        
        
          //This code specifies the output syntax for the summary data
          try {
      	        FileOutputStream fos = new FileOutputStream(summaryDataFile, true);		  
                PrintWriter out = new PrintWriter(fos);

                out.print(runNumber);
	            out.print("     ");

	            out.print(getRandomSeed());
	            out.print("     ");

                out.print(getIteration());
                out.print("     ");
                
                //CUMULATIVE OVER RUN
                //% COOPOWN
                out.print((double) ((double) phenotypeStats[COOPOWN]/(double) phenotypeStats[MEETOWN])); //% coopOwn
                out.print("     ");
                
                //% DEFOTHER
                out.print((double) ((double) phenotypeStats[DEFOTHER]/(double) phenotypeStats[MEETOTHER])); //% defOther
                out.print("     ");
                
                //% consisEthno
                out.print((double) (((double) phenotypeStats[DEFOTHER]) + ((double) phenotypeStats[COOPOWN])) / (((double) phenotypeStats[MEETOWN]) + ((double) phenotypeStats[MEETOTHER])));
                out.print("     ");
                
                //% meetOwn
                out.print((double) (((double) phenotypeStats[MEETOWN])/(((double) phenotypeStats[MEETOWN]) + ((double) phenotypeStats[MEETOTHER]))));
                out.print("     ");
                
                //% C
                out.print((double) (((double) phenotypeStats[COOPOWN]) + ((double) phenotypeStats[COOPOTHER])) / (((double) phenotypeStats[MEETOWN]) + ((double) phenotypeStats[MEETOTHER])));
                out.print("     ");
                
           
                //LASTHUNDRED
                //% COOPOWN lastHundred
                out.print((double) ((double) lasthundredPhenotypeStats[COOPOWN]/(double) lasthundredPhenotypeStats[MEETOWN])); //% coopOwn
                out.print("     ");
                
                //% DEFOTHER lastHundred
                out.print((double) ((double) lasthundredPhenotypeStats[DEFOTHER]/(double) lasthundredPhenotypeStats[MEETOTHER])); //% defOther
                out.print("     ");
                
                //% consisEthno lastHundred
                out.print((double) (((double) lasthundredPhenotypeStats[DEFOTHER]) + ((double) lasthundredPhenotypeStats[COOPOWN])) / (((double) lasthundredPhenotypeStats[MEETOWN]) + ((double) lasthundredPhenotypeStats[MEETOTHER])));
                out.print("     ");
                
                //% meetOwn
                out.print((double) ((double) lasthundredPhenotypeStats[MEETOWN]/((double) lasthundredPhenotypeStats[MEETOWN] + (double) lasthundredPhenotypeStats[MEETOTHER])));
                out.print("     ");
                
                //% C
                out.print((double) ((double) lasthundredPhenotypeStats[COOPOWN] + (double) lasthundredPhenotypeStats[COOPOTHER]) / ((double) lasthundredPhenotypeStats[MEETOWN] + (double) lasthundredPhenotypeStats[MEETOTHER]));
                out.print("     ");
                
                
	            //Genotypes over 100 final periods
	            for (int i=0;i<4;i++){
	                out.print(lasthundredGenotypeStats[i]);
	                out.print("     ");
	            }     
	            
	            
	            out.println();
	            
	            out.close();
	     
        	    } catch (FileNotFoundException e) {
		        	System.err.println("FileStreamsTest: "+ e);
	        	} catch (IOException e) {
			        System.err.println("FileStreamsTest: "+ e);
		        }	

		  
		}
              
        }
	    
	}
	
	    
	//SOME GETTERS and SETTERS for various switches
	//(These makes the global settings accessible from each player's code)
	
	public boolean getOneTagOnly() {
	    return useOneTagOnly;
	}

    public int getStopIteration() {
        return stopAtIteration;
    }

	public boolean getLocateChildNear() {
	    return locateChildNearParent;
	}
	
	public void incrementPhenotype(int _type) {
	    phenotypeStats[_type]++;
	    if (getIteration() >= (stopAtIteration-100)) {
	        lasthundredPhenotypeStats[_type]++;
	    }
	}
	
    public int getNumPlayers() {
        return agents.getSize();
    }

    public int getChildNearParentZeroFalse() {
        return (childNearParentZeroFalse);
    }
    public void setChildNearParentZeroFalse(int _value) {
        childNearParentZeroFalse = _value;
    }
    
    
    //part of the initialization routine each run
    public void onSetup() {
        
        super.onSetup();
        
        //test if the model has run enough times for the current
        //experiment... if so, exit
        
        if (runNumber == runMax) {
            scape.exit();
        }
        else { //otherwise...
        
        //Set up new empty vector to hold agents as they are added
	//	((ScapeVector) agents).setExtent(new Coordinate1DDiscrete(0));
		((ScapeVector) agents).setExtent(new Coordinate1DDiscrete(latticeWidth*latticeHeight));
		
		
		if (colorSetting == 2) {
			useMoreColors = false;
		}
		else {
			useMoreColors = true;
		}
		
		if (colorSetting == 1) {
			useOneTagOnly = true;
		}
		else {
			useOneTagOnly = false;
		}
		
		
		//Initialize "clone near parent" switch
		if (childNearParentZeroFalse == 0) {
		    locateChildNearParent = false;
		}
		else {
		    locateChildNearParent = true;
		}
		
		//Reset stats
		for (int i=0;i<phenotypeStats.length;i++) {
	        phenotypeStats[i] = 0;
	        lasthundredPhenotypeStats[i] = 0;
	    }
	    
	    for (int i=0;i<4;i++) {
	        lasthundredGenotypeStats[i] = 0;
	    }
	
	    //increment the number of runs completed so far
	    runNumber++;
		
		System.out.println();
		System.out.print("RUN #");
		System.out.println(runNumber);
		System.out.println("******************************************");
	    
	    }
	}        
        
    
    //MORE GETTERS and SETTERS
    //(These allow abstraction of variables so they are passed back and forth
    // rather than being directly accessed across different pieces of the
    // program)
    
    public boolean getUseMoreColors() {
        return (useMoreColors);
    }
 
    public int getMaxAgents() {
        return (latticeWidth*latticeHeight);
    }
    
    public int getNumColors() {
    	return (colorSetting);
    }

	public boolean getUseCombat() {
		return (combatOn);	
	}

	public boolean getUseDiscrimCost() {
		return (discrimCostOn);
	}

	public double getDiscrimCost() {
		return (discrimCost);
	}

    public boolean getUseNoise() {
        return (noiseOn);
    }
    public double getNoiseLevel() {
        return (noiseLevel);
    }
    
    public void setPlayMode(int value) {
        playMode = value;
    }    
    
    public int getLatticeWidth() {
        return latticeWidth;
    }

    public void setLatticeWidth(int _latticeWidth) {
        latticeWidth = _latticeWidth;
    }

    public int getLatticeHeight() {
        return latticeHeight;
    }

    public void setLatticeHeight(int _latticeHeight) {
        latticeHeight = _latticeHeight;
    }
    public double getMutationRate() {
        return mutationRate;
    }
    public void setMutationRate(double mutationRate) {
        this.mutationRate = mutationRate;
    }
    
    public void incrementCountAfterImm() {
    	countAfterImm++;
    	
    }
    public void incrementCountAfterFission() {
    	countAfterFission++;
    }
    
    public int getColorSetting() {
    	return (colorSetting);
    }
    public void setColorSetting(int _setting) {
    	colorSetting = _setting;
    }

    public double getDeathRate() {
        return deathRate;
    }
    public void setDeathRate(double _deathRate) {
        deathRate = _deathRate;
    }
       
    public double getBaseOTR() {
        return baseOTR;
    }
    public void setBaseOTR(double _OTR) {
        baseOTR = _OTR;
    }
       
    public double getCostOfGiving() {
        return costOfGiving;
    }
    public void setCostOfGiving(double _cost) {
        costOfGiving = _cost;
    }
    
    public double getGainOfReceiving() {
        return gainOfReceiving;
    }
    public void setGainOfReceiving(double _gain) {
        gainOfReceiving = _gain;
    }
}